@@ -4,5 +4,12 @@ from multibot.models.database import *
 from multibot.models.enums import *
 from multibot.models.event_component import *
 from multibot.models.message import *
+from multibot.models.mute import *
 from multibot.models.registered_callback import *
+from multibot.models.role import *
 from multibot.models.user import *
+
+try:
+    from models import *
+except ModuleNotFoundError:
+    pass
